package cn.newfeifan.mall.module.distri.service.socialstatuschangecategory;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.socialstatuschangecategory.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.socialstatuschangecategory.SocialStatusChangeCategoryDO;
import cn.newfeifan.mall.module.distri.dal.mysql.socialstatuschangecategory.SocialStatusChangeCategoryMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link SocialStatusChangeCategoryServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(SocialStatusChangeCategoryServiceImpl.class)
public class SocialStatusChangeCategoryServiceImplTest extends BaseDbUnitTest {

    @Resource
    private SocialStatusChangeCategoryServiceImpl socialStatusChangeCategoryService;

    @Resource
    private SocialStatusChangeCategoryMapper socialStatusChangeCategoryMapper;

    @Test
    public void testCreateSocialStatusChangeCategory_success() {
        // 准备参数
        SocialStatusChangeCategorySaveReqVO createReqVO = randomPojo(SocialStatusChangeCategorySaveReqVO.class).setId(null);

        // 调用
        Long socialStatusChangeCategoryId = socialStatusChangeCategoryService.createSocialStatusChangeCategory(createReqVO);
        // 断言
        assertNotNull(socialStatusChangeCategoryId);
        // 校验记录的属性是否正确
        SocialStatusChangeCategoryDO socialStatusChangeCategory = socialStatusChangeCategoryMapper.selectById(socialStatusChangeCategoryId);
        assertPojoEquals(createReqVO, socialStatusChangeCategory, "id");
    }

    @Test
    public void testUpdateSocialStatusChangeCategory_success() {
        // mock 数据
        SocialStatusChangeCategoryDO dbSocialStatusChangeCategory = randomPojo(SocialStatusChangeCategoryDO.class);
        socialStatusChangeCategoryMapper.insert(dbSocialStatusChangeCategory);// @Sql: 先插入出一条存在的数据
        // 准备参数
        SocialStatusChangeCategorySaveReqVO updateReqVO = randomPojo(SocialStatusChangeCategorySaveReqVO.class, o -> {
            o.setId(dbSocialStatusChangeCategory.getId()); // 设置更新的 ID
        });

        // 调用
        socialStatusChangeCategoryService.updateSocialStatusChangeCategory(updateReqVO);
        // 校验是否更新正确
        SocialStatusChangeCategoryDO socialStatusChangeCategory = socialStatusChangeCategoryMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, socialStatusChangeCategory);
    }

    @Test
    public void testUpdateSocialStatusChangeCategory_notExists() {
        // 准备参数
        SocialStatusChangeCategorySaveReqVO updateReqVO = randomPojo(SocialStatusChangeCategorySaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> socialStatusChangeCategoryService.updateSocialStatusChangeCategory(updateReqVO), SOCIAL_STATUS_CHANGE_CATEGORY_NOT_EXISTS);
    }

    @Test
    public void testDeleteSocialStatusChangeCategory_success() {
        // mock 数据
        SocialStatusChangeCategoryDO dbSocialStatusChangeCategory = randomPojo(SocialStatusChangeCategoryDO.class);
        socialStatusChangeCategoryMapper.insert(dbSocialStatusChangeCategory);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbSocialStatusChangeCategory.getId();

        // 调用
        socialStatusChangeCategoryService.deleteSocialStatusChangeCategory(id);
       // 校验数据不存在了
       assertNull(socialStatusChangeCategoryMapper.selectById(id));
    }

    @Test
    public void testDeleteSocialStatusChangeCategory_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> socialStatusChangeCategoryService.deleteSocialStatusChangeCategory(id), SOCIAL_STATUS_CHANGE_CATEGORY_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetSocialStatusChangeCategoryPage() {
       // mock 数据
       SocialStatusChangeCategoryDO dbSocialStatusChangeCategory = randomPojo(SocialStatusChangeCategoryDO.class, o -> { // 等会查询到
           o.setName(null);
           o.setCode(null);
           o.setVariationValue(null);
           o.setDescription(null);
           o.setCreateTime(null);
       });
       socialStatusChangeCategoryMapper.insert(dbSocialStatusChangeCategory);
       // 测试 name 不匹配
       socialStatusChangeCategoryMapper.insert(cloneIgnoreId(dbSocialStatusChangeCategory, o -> o.setName(null)));
       // 测试 code 不匹配
       socialStatusChangeCategoryMapper.insert(cloneIgnoreId(dbSocialStatusChangeCategory, o -> o.setCode(null)));
       // 测试 variationValue 不匹配
       socialStatusChangeCategoryMapper.insert(cloneIgnoreId(dbSocialStatusChangeCategory, o -> o.setVariationValue(null)));
       // 测试 description 不匹配
       socialStatusChangeCategoryMapper.insert(cloneIgnoreId(dbSocialStatusChangeCategory, o -> o.setDescription(null)));
       // 测试 createTime 不匹配
       socialStatusChangeCategoryMapper.insert(cloneIgnoreId(dbSocialStatusChangeCategory, o -> o.setCreateTime(null)));
       // 准备参数
       SocialStatusChangeCategoryPageReqVO reqVO = new SocialStatusChangeCategoryPageReqVO();
       reqVO.setName(null);
       reqVO.setCode(null);
       reqVO.setVariationValue(null);
       reqVO.setDescription(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<SocialStatusChangeCategoryDO> pageResult = socialStatusChangeCategoryService.getSocialStatusChangeCategoryPage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbSocialStatusChangeCategory, pageResult.getList().get(0));
    }

}