package cn.newfeifan.mall.module.distri.service.usercollectbefore;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.mock.mockito.MockBean;

import javax.annotation.Resource;

import cn.newfeifan.mall.framework.test.core.ut.BaseDbUnitTest;

import cn.newfeifan.mall.module.distri.controller.admin.usercollectbefore.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.usercollectbefore.UserCollectBeforeDO;
import cn.newfeifan.mall.module.distri.dal.mysql.usercollectbefore.UserCollectBeforeMapper;
import cn.newfeifan.mall.framework.common.pojo.PageResult;

import javax.annotation.Resource;
import org.springframework.context.annotation.Import;
import java.util.*;
import java.time.LocalDateTime;

import static cn.hutool.core.util.RandomUtil.*;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;
import static cn.newfeifan.mall.framework.test.core.util.AssertUtils.*;
import static cn.newfeifan.mall.framework.test.core.util.RandomUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.LocalDateTimeUtils.*;
import static cn.newfeifan.mall.framework.common.util.object.ObjectUtils.*;
import static cn.newfeifan.mall.framework.common.util.date.DateUtils.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

/**
 * {@link UserCollectBeforeServiceImpl} 的单元测试类
 *
 * @author 非繁人
 */
@Import(UserCollectBeforeServiceImpl.class)
public class UserCollectBeforeServiceImplTest extends BaseDbUnitTest {

    @Resource
    private UserCollectBeforeServiceImpl userCollectBeforeService;

    @Resource
    private UserCollectBeforeMapper userCollectBeforeMapper;

    @Test
    public void testCreateUserCollectBefore_success() {
        // 准备参数
        UserCollectBeforeSaveReqVO createReqVO = randomPojo(UserCollectBeforeSaveReqVO.class).setId(null);

        // 调用
        Long userCollectBeforeId = userCollectBeforeService.createUserCollectBefore(createReqVO);
        // 断言
        assertNotNull(userCollectBeforeId);
        // 校验记录的属性是否正确
        UserCollectBeforeDO userCollectBefore = userCollectBeforeMapper.selectById(userCollectBeforeId);
        assertPojoEquals(createReqVO, userCollectBefore, "id");
    }

    @Test
    public void testUpdateUserCollectBefore_success() {
        // mock 数据
        UserCollectBeforeDO dbUserCollectBefore = randomPojo(UserCollectBeforeDO.class);
        userCollectBeforeMapper.insert(dbUserCollectBefore);// @Sql: 先插入出一条存在的数据
        // 准备参数
        UserCollectBeforeSaveReqVO updateReqVO = randomPojo(UserCollectBeforeSaveReqVO.class, o -> {
            o.setId(dbUserCollectBefore.getId()); // 设置更新的 ID
        });

        // 调用
        userCollectBeforeService.updateUserCollectBefore(updateReqVO);
        // 校验是否更新正确
        UserCollectBeforeDO userCollectBefore = userCollectBeforeMapper.selectById(updateReqVO.getId()); // 获取最新的
        assertPojoEquals(updateReqVO, userCollectBefore);
    }

    @Test
    public void testUpdateUserCollectBefore_notExists() {
        // 准备参数
        UserCollectBeforeSaveReqVO updateReqVO = randomPojo(UserCollectBeforeSaveReqVO.class);

        // 调用, 并断言异常
        assertServiceException(() -> userCollectBeforeService.updateUserCollectBefore(updateReqVO), USER_COLLECT_BEFORE_NOT_EXISTS);
    }

    @Test
    public void testDeleteUserCollectBefore_success() {
        // mock 数据
        UserCollectBeforeDO dbUserCollectBefore = randomPojo(UserCollectBeforeDO.class);
        userCollectBeforeMapper.insert(dbUserCollectBefore);// @Sql: 先插入出一条存在的数据
        // 准备参数
        Long id = dbUserCollectBefore.getId();

        // 调用
        userCollectBeforeService.deleteUserCollectBefore(id);
       // 校验数据不存在了
       assertNull(userCollectBeforeMapper.selectById(id));
    }

    @Test
    public void testDeleteUserCollectBefore_notExists() {
        // 准备参数
        Long id = randomLongId();

        // 调用, 并断言异常
        assertServiceException(() -> userCollectBeforeService.deleteUserCollectBefore(id), USER_COLLECT_BEFORE_NOT_EXISTS);
    }

    @Test
    @Disabled  // TODO 请修改 null 为需要的值，然后删除 @Disabled 注解
    public void testGetUserCollectBeforePage() {
       // mock 数据
       UserCollectBeforeDO dbUserCollectBefore = randomPojo(UserCollectBeforeDO.class, o -> { // 等会查询到
           o.setProductSpuId(null);
           o.setUserId(null);
           o.setCreateTime(null);
       });
       userCollectBeforeMapper.insert(dbUserCollectBefore);
       // 测试 productSpuId 不匹配
       userCollectBeforeMapper.insert(cloneIgnoreId(dbUserCollectBefore, o -> o.setProductSpuId(null)));
       // 测试 userId 不匹配
       userCollectBeforeMapper.insert(cloneIgnoreId(dbUserCollectBefore, o -> o.setUserId(null)));
       // 测试 createTime 不匹配
       userCollectBeforeMapper.insert(cloneIgnoreId(dbUserCollectBefore, o -> o.setCreateTime(null)));
       // 准备参数
       UserCollectBeforePageReqVO reqVO = new UserCollectBeforePageReqVO();
       reqVO.setProductSpuId(null);
       reqVO.setUserId(null);
       reqVO.setCreateTime(buildBetweenTime(2023, 2, 1, 2023, 2, 28));

       // 调用
       PageResult<UserCollectBeforeDO> pageResult = userCollectBeforeService.getUserCollectBeforePage(reqVO);
       // 断言
       assertEquals(1, pageResult.getTotal());
       assertEquals(1, pageResult.getList().size());
       assertPojoEquals(dbUserCollectBefore, pageResult.getList().get(0));
    }

}